package DBAgent;

import alice.tucson.api.*;

import java.util.Vector;
import alice.logictuple.*;
import java.io.*;
import SQLIstructionSet.*;

/**
 * Agente che si occupa della pre-elaborazione delle query
 * @author luca vanzolini
 */
public class PreElaboration extends Agent{
	
	/**
	 * Centro tuple sopra il quale l'agente effettua la pre-elaborazione delle query
	 */
	TupleCentreId TCid;
	
	/**
	 * Costruttore
	 * @param tcId identificatore del centro tuple (coda)
	 */
	public PreElaboration(AgentId name, TupleCentreId tcId) throws TucsonException{
		super(name);
		TCid = tcId;
	}
	
	/**
	 * Corpo del thread
	 */
	public void body(){
		try{
			while(true){
				LogicTuple req = new LogicTuple("query", new Var("QueryS"), new Var("Uid"), new Var("Tid"), new Var("Qid"));
				LogicTuple reqE = in(TCid,req);	
				TupleArgument Qid = reqE.getArg(3);	
				TupleArgument Q = reqE.getArg(0);	
				String QS = Q.toString();
				QS = QS.substring(1,QS.length()-1);
				TupleArgument Uid = reqE.getArg(1);	
				TupleArgument Tid = reqE.getArg(2);	
				LogicTuple newReq = eval(QS,Uid.intValue(),Tid.intValue(),Qid.intValue());
				out(TCid,newReq);	
			}
		}catch(Exception e){}
	}
	
	/**
	 * Medoto che effettua la pre-elaborazione delle query
	 * @param query Oggetto query
	 * @param Uid ID dell'utente che ah inserito la query
	 * @param Tid ID del terminale su cui si trova l'utente identificato da Uid
	 * @return restituisce il risultato della pre-elaborazione della query
	 */
	private LogicTuple eval(String query, int Uid, int Tid, int Qid){
		LogicTuple result;
		try{
			SQLIstruction queryO = (SQLIstruction)this.convertToObject(query);
			if(queryO.getType() == "sel"){
				Vector From = ((Select)queryO).from;
				Vector Where = ((Select)queryO).where;
				TupleArgument ArgF[] = new TupleArgument[From.size()];
				for(int i=0; i<From.size(); i++)
					ArgF[i] = new Value(((String)From.elementAt(i)));	
				TupleArgument ArgW[] = new TupleArgument[Where.size()];
				for(int i=0; i<Where.size(); i++)
					ArgW[i] = new Value(((String)Where.elementAt(i)));	
				result = new LogicTuple("query_e", new Value(query), new Value(Uid), new Value(Tid), new Value(Qid), new Value(ArgF), new Value(ArgW));	
			}
			else
				result = new LogicTuple("query_e", new Value(query), new Value(Uid), new Value(Tid), new Value(Qid), new Value(new TupleArgument[0]), new Value(new TupleArgument[0]));
		}catch(Exception e){
			result = new LogicTuple("error", new Value(Uid), new Value(Tid), new Value(Qid), new Value("Query non valida"));	
		}
		return result;
	}
	
	/**
	 * Metodo che crea un oggetto dalla sua serializzazione
	 * @param queryS Oggetto serializzato
	 * @return Oggetto
	 */
	public Object convertToObject(String queryS){
		Object o = null;
		try{		
			byte[] buffer = queryS.getBytes();
			ByteArrayInputStream BAIS = new ByteArrayInputStream(buffer);
			ObjectInputStream OIS = new ObjectInputStream(BAIS);
			o = OIS.readObject();
		}catch(Exception e){
			System.out.println("Error in convertToObject");
		}
		return o;
	}	
	
}
