package DBAgent;

import alice.tucson.api.*;
import alice.logictuple.*;

import java.util.Vector;

/**
 * Agente che si occupa dello smistamento delle query in coda
 * @author luca vanzolini
 */
public class QueryOptAgent implements IQueryOptAgent{
	
	/**
	 * centro tuple "coda"
	 */
	TupleCentreId tcCoda;
	
	/**
	 * centro tuple "query"
	 */
	TupleCentreId tcQuery;
	
	/**
	 * centro tuple "sinc"
	 */
	TupleCentreId tcSinc;
	
	/**
	 * centro tuple "sinc"
	 */
	TupleCentreId tcLocalSinc;	
	
	/**
	 * identificativo del terminale
	 */
	int Tid;
	
	/**
	 * identificativo del terminale
	 */
	String TName;
	
	/**
	 * Contesto di lavoro tucson
	 */	
	TucsonContext ctx;
	
	/**
	 * Vettore contenente i riferimenti alle code
	 */
	Vector TCCodaVector;
	
	/**
	 * Vettore contenente i riferimenti al centro tuple query_in_esecuzione
	 */
	Vector TCQueryVector;
	
	/**
	 * Vettore contenente i riferimenti ai terminali
	 */
	Vector NameVector;

	/**
	 * Vettore contenente i riferimenti ai thread in ascolto sulle code
	 */
	Vector ThreadCVector;
	
	/**
	 * Vettore contenente i riferimenti ai thread in ascolto sul centro tuple query_in_esecuzione
	 */
	Vector ThreadQVector;

	/**
	 * Vettore contenente le query in attesa di essere elaborate
	 */
	Vector SuspendQuery;
	
	LogicTuple NewOptQuery;	
	
	AccessAgent AA;
	
	RemoveAgent RA;
	
	QueryOpt Opt;
	
	QueryExecute Exe;
	
	
	/**
	 * Costruttore
	 * @param tcidCoda centro tuple "coda"
	 * @param tcidQuery centro tuple "query"
	 * @param name identificativo del terminale
	 * @param Tid identificativo del terminale
	 */
	public QueryOptAgent(TupleCentreId tcidCoda, TupleCentreId tcidQuery, TupleCentreId tcidSinc, String name, int Tid, TupleCentreId LS){
		super();
		tcCoda = tcidCoda;
		tcQuery = tcidQuery;
		tcSinc = tcidSinc;
		tcLocalSinc = LS;
		TName = name;
		this.Tid = Tid;
		try{
			ctx = Tucson.enterDefaultContext();
		}catch(Exception e){}
		TCCodaVector = new Vector();
		TCQueryVector = new Vector();
		NameVector = new Vector();
		SuspendQuery = new Vector();
		ThreadCVector = new Vector();
		try{
			LogicTuple sem = new LogicTuple("QuerySinc", new Value(0));
			ctx.out(tcLocalSinc,sem);			
			
			AA = new AccessAgent(new AgentId("alice"),tcSinc,this,Tid);
			AA.spawn();
			RA = new RemoveAgent(new AgentId("alice"),tcSinc,this,Tid);
			RA.spawn();
			
			ctx.out(tcSinc, new LogicTuple("addTerminal", new Value(name), new Value(Tid)));
			
			Opt = new QueryOpt(new AgentId("alice"),this,tcLocalSinc);
			Opt.spawn();
			Exe = new QueryExecute(new AgentId("alice"),Tid,this,tcLocalSinc,tcSinc);
			Exe.spawn();
		}catch(Exception e){
			System.out.println("Error");
		}
	}
	
	/**
	 * Metodo che permette di notificare all'agente queryOpt l'ingresso nel sistema di un 
	 * nuovo terminale
	 * @param tcidCoda centro tuple "coda"
	 * @param tcidQuery centro tuple "query"
	 * @param name identificativo del terminale
	 */
	public void addTerminal(TupleCentreId tcidCoda, TupleCentreId tcidQuery, String name){
		TCCodaVector.add(tcidCoda);
		TCQueryVector.add(tcidQuery);
		this.NameVector.add(name);
		this.ThreadCVector.add(new QueryListener(Tid,this,tcidCoda,name));
		LogicTuple LT = new LogicTuple("reg", new Value(Tid));
		try{
			ctx.out(tcidCoda,LT);
		}catch(Exception e){
			System.out.println("Iscription Error");
		}
	}
	
	/**
	 * Metodo che permette di notificare all'agente queryOpt l'uscita dal sistema di un 
	 * terminale
	 * @param name identificativo del terminale che si scollega dal sistema
	 */	
	public void removeTerminal(String name){
		int ind = PosisionToName(name);
		LogicTuple LT = new LogicTuple("unreg", new Value(Tid));
		try{
			ctx.out((TupleCentreId)TCCodaVector.elementAt(ind),LT);
		}catch(Exception e){}
		TCCodaVector.removeElementAt(ind);
		TCQueryVector.removeElementAt(ind);
		NameVector.removeElementAt(ind);
		((QueryListener)ThreadCVector.elementAt(ind)).stop();
		ThreadCVector.removeElementAt(ind);
	}
	
	/**
	 * Metodo che mi dice in che posizione si trova name nel vettore dei nomi dei terminali
	 * @param name nome del terminale
	 * @return posizione nei vettori del terminale
	 */
	private int PosisionToName(String name){
		int ind = 0;
		boolean condizione = true;
		while(ind < NameVector.size() && condizione){
			if((String)(NameVector.elementAt(ind)) == name)
				condizione = false;
			else
				ind++;
		}
		return ind;
	}
	
	/**
	 * Metodo che permette la notifica di una nuova query su una qualsiasi coda
	 * @param query Query
	 * @param name nome del terminale da cui  stata l'anciata la query
	 */
	public void newQery(LogicTuple query, String name){
		SuspendQuery.add(query);
		LogicTuple t = new LogicTuple("newQuery");
		try{
			ctx.out(tcLocalSinc,t);
		}catch(Exception e){}
		System.out.println("Query pervenuta");
	}
	
	/**
	 * 
	 * @param Tid
	 * @return
	 */
	public String GetHostName(int T){
		try{
			LogicTuple t = new LogicTuple("convertToString", new Value(T), new Value(this.Tid));
			ctx.out(tcSinc,t);
			LogicTuple r = new LogicTuple("convResult", new Var("HName"), new Value(this.Tid));
			ctx.in(tcSinc,r);
			TupleArgument TAName = r.getArg(0);
			return TAName.toString();
		}catch(Exception e){}
		return "";
	}
	
	/**
	 * 
	 */
	public void SetNewOptQuery(LogicTuple querys){
		NewOptQuery = querys;
	}
	
	/**
	 * 
	 */
	public LogicTuple GetNewOptQuery(){
		return NewOptQuery;
	}
	
	/**
	 * 
	 */
	public Vector GetSuspendQuery(){
		return SuspendQuery;
	}
	
	/**
	 * 
	 */
	public Vector GetTCQueryVector(){
		return TCQueryVector;
	}
	
	/**
	 * 
	 */
	public Vector GetNameHostVector(){
		return NameVector;
	}	
	
	
	/**
	 * 
	 */
	public void UpdateNewOptQuery(int Qid){
		try{
			NewOptQuery = null;
			boolean cond = true;
			int i = 0;
			while(cond){
				LogicTuple query = (LogicTuple)SuspendQuery.elementAt(i);
				TupleArgument Q = query.getArg(3);
				if(Q.intValue() == Qid){
					cond = false;
					SuspendQuery.removeElementAt(i);
				}
				
				i++;
				if(i == SuspendQuery.size())
					cond = false;
			}
			
			LogicTuple go = new LogicTuple("newQuery");
			if(SuspendQuery.size() != 0)
				ctx.out(tcLocalSinc,go);
		}catch(Exception e){}
	}
	
	/**
	 * 
	 */
	public void removeQuery(int Qid){
		try{
			NewOptQuery = null;
			boolean cond = true;
			int i = 0;
			while(cond){
				LogicTuple query = (LogicTuple)SuspendQuery.elementAt(i);
				TupleArgument Q = query.getArg(3);
				if(Q.intValue() == Qid){
					cond = false;
					SuspendQuery.removeElementAt(i);
				}
				
				i++;
				if(i == SuspendQuery.size())
					cond = false;
			}
		}catch(Exception e){}
	}

}
