package DBAgent;

import alice.tucson.api.*;
import alice.logictuple.*;

public class TC_Library {

	static TupleCentreId newCoda(String name){
		TupleCentreId coda = null;
		try{
			coda = new TupleCentreId(name);
			TucsonContext ctx = Tucson.enterDefaultContext();
			ctx.out(coda,new LogicTuple("n_asc", new Value(0)));	
			
			String reactions = 
			"reaction(out(reg(T)),(\n" +
			"			in_r(reg(T)),\n" +
			"			in_r(n_asc(N)),\n" +
			"			N1 is N + 1,\n" +
			"			out_r(n_asc(N1)),\n" +
			"			out_r(asc(N,T))\n" +
			"			)).\n\n" +
						
			"reaction(out(query_e(Q,U,T,QID,F,W)),(\n" +
			"			in_r(query_e(Q,U,T,QID,F,W)),\n" +
			"			rd_r(n_asc(N)),\n" +
			"			out_r(loopQuery(Q,U,T,QID,F,W,N))\n" +
			"			)).\n\n" +

			"reaction(out_r(loopQuery(Q,U,T,QID,F,W,0)),(\n" +
			"			in_r(loopQuery(Q,U,T,QID,F,W,0)))).\n\n" +			
						
			"reaction(out_r(loopQuery(Q,U,T,QID,F,W,N)),(\n" +
			"			in_r(loopQuery(Q,U,T,QID,F,W,N)),\n" +
			"			N1 is N - 1,\n" +
			"			rd_r(asc(N1,TID)),\n" +
			"			out_r(query_e(Q,U,T,QID,F,W,TID)),\n" +
			"			out_r(loopQuery(Q,U,T,QID,F,W,N1))\n" +
			"			)).\n\n" +
						
			"reaction(out(unreg(T)),(\n" +
			"			in_r(unreg(T)),\n" +
			"			rd_r(asc(N,T)),\n" +
			"			in_r(n_asc(NT)),\n" +
			"			N1 is NT - 1,\n" +
			"			out_r(n_asc(N1)),\n" +
			"			S is N + 1,\n" +
			"			out_r(loopUnreg(S,N1))\n" +
			"			)).\n\n" +

			"reaction(out_r(loopUnreg(X,X)),(\n" +
			"			in_r(loopUnreg(X,X)),\n" +
			"			X1 is X - 1,\n" +
			"			in_r(asc(X1,_)),\n" +
			"			in_r(asc(X,T)),\n" +
			"			out_r(asc(X1,T))\n" +
			"			)).\n\n" +
						
			"reaction(out_r(loopUnreg(X,Y)),(\n" +
			"			Y > X,\n" +
			"			in_r(loopUnreg(X,Y)),\n" +
			"			X1 is X - 1,\n" +
			"			in_r(asc(X1,_)),\n" +
			"			in_r(asc(X,T)),\n" +
			"			out_r(asc(X1,T)),\n" +
			"			X2 is X + 1,\n" +
			"			out_r(loopUnreg(X2,Y))\n" +
			"			)).\n\n" +
						
			"reaction(out_r(loopUnreg(X,Y)),(\n" +
			"			Y < X,\n" +
			"			N is X - 1,\n" +
			"			in_r(asc(N,T))\n" + 
			"			)).\n\n";
			
			ctx.setSpec(coda,reactions);
		}catch(Exception e){
			System.out.println("ReSpecT Error");
		}
		return coda;
	}
	
	static TupleCentreId newSinc(String name){
		TupleCentreId coda = null;
		try{
			coda = new TupleCentreId(name);
			TucsonContext ctx = Tucson.enterDefaultContext();
			ctx.out(coda,new LogicTuple("n_asc", new Value(0)));	
			ctx.out(coda,new LogicTuple("Ready"));	
			
			String reactions = 
			"reaction(out_r(loopA(Name,0,T)),(\n" +
			"			in_r(loopA(Name,0,T)))).\n\n" +			
							
			"reaction(out_r(loopA(Name,N,T)),(\n" +
			"			in_r(loopA(Name,N,T)),\n" +
			"			N1 is N - 1,\n" +
			"			rd_r(asc(N1,TID,NameP)),\n" +
			"			out_r(addT(Name,TID)),\n" +
			"			out_r(addT(NameP,T)),\n" +
			"			out_r(loopA(Name,N1,T))\n" +
			"			)).\n\n" +			

			"reaction(out_r(loopR(Name,0)),(\n" +
			"			in_r(loopR(Name,0)))).\n\n" +			
							
			"reaction(out_r(loopR(Name,N)),(\n" +
			"			in_r(loopR(Name,N)),\n" +
			"			N1 is N - 1,\n" +
			"			rd_r(asc(N1,TID,NameP)),\n" +
			"			out_r(remT(Name,TID)),\n" +
			"			out_r(loopR(Name,N1))\n" +
			"			)).\n\n" +	
				
			"reaction(out(addTerminal(Name,T)),(\n" +
			"			in_r(addTerminal(Name,T)),\n" +
			"			in_r(n_asc(N)),\n" +
			"			N1 is N + 1,\n" +
			"			out_r(n_asc(N1)),\n" +
			"			out_r(asc(N,T,Name)),\n" +
			"			out_r(addT(Name,T)),\n" +
			"			out_r(loopA(Name,N,T))\n" +
			"			)).\n\n" +
						
			"reaction(out(remTerminal(T)),(\n" +
			"			in_r(remTerminal(T)),\n" +
			"			rd_r(asc(N,T,_)),\n" +
			"			in_r(n_asc(NT)),\n" +
			"			N1 is NT - 1,\n" +
			"			out_r(n_asc(N1)),\n" +
			"			S is N + 1,\n" +
			"			out_r(loopUnreg(S,N1)),\n" +
			"			out_r(loopR(Name,N))\n" +
			"			)).\n\n" +

			"reaction(out_r(loopUnreg(X,X)),(\n" +
			"			in_r(loopUnreg(X,X)),\n" +
			"			X1 is X - 1,\n" +
			"			in_r(asc(X1,_,_)),\n" +
			"			in_r(asc(X,T,_)),\n" +
			"			out_r(asc(X1,T,_))\n" +
			"			)).\n\n" +
						
			"reaction(out_r(loopUnreg(X,Y)),(\n" +
			"			Y > X,\n" +
			"			in_r(loopUnreg(X,Y)),\n" +
			"			X1 is X - 1,\n" +
			"			in_r(asc(X1,_,_)),\n" +
			"			in_r(asc(X,T,_)),\n" +
			"			out_r(asc(X1,T,_)),\n" +
			"			X2 is X + 1,\n" +
			"			out_r(loopUnreg(X2,Y))\n" +
			"			)).\n\n" +
						
			"reaction(out_r(loopUnreg(X,Y)),(\n" +
			"			Y < X,\n" +
			"			N is X - 1,\n" +
			"			in_r(asc(N,T,_))\n" + 
			"			)).\n\n" +
			
			"reaction(out(convertToString(TidS,TidC)),(\n" +
			"			in_r(convertToString(TidS,TidC)),\n" +
			"			rd_r(asc(N,TidS,Name)),\n" +
			"			out_r(convResult(Name,TidC))\n" +
			"			)).\n\n" +	
				
			"reaction(out(removeQuery(Q,T)),(\n" +
			"			in_r(removeQuery(Q,T)),\n" +
			"			rd_r(n_asc(N)),\n" +
			"			N1 is N - 1,\n" +
			"			out_r(loopQ(Q,T,N1))\n" +
			"			)).\n\n" +
			
			"reaction(out_r(loopQ(Q,T,0)),(\n" +
			"			in_r(loopQ(Q,T,0)),\n" +
			"			rd_r(asc(0,Tid,_)),\n" +
			"			T = Tid\n" +
			"			)).\n\n" +
			
			"reaction(out_r(loopQ(Q,T,0)),(\n" +
			"			in_r(loopQ(Q,T,0)),\n" +
			"			rd_r(asc(0,Tid,_)),\n" +
			"			out_r(remQuery(Q,Tid))\n" +
			"			)).\n\n" +
			
			"reaction(out_r(loopQ(Q,T,N)),(\n" +
			"			in_r(loopQ(Q,T,N)),\n" +
			"			rd_r(asc(N,Tid,_)),\n" +
			"			T = Tid,\n" +
			"			N1 is N - 1,\n" +
			"			out_r(loopQ(Q,T,N1))\n" +
			"			)).\n\n" +
			
			"reaction(out_r(loopQ(Q,T,N)),(\n" +
			"			in_r(loopQ(Q,T,N)),\n" +
			"			rd_r(asc(N,Tid,_)),\n" +
			"			out_r(remQuery(Q,Tid)),\n" +
			"			N1 is N - 1,\n" +
			"			out_r(loopQ(Q,T,N1))\n" +
			"			)).\n\n";
			
			ctx.setSpec(coda,reactions);
		}catch(Exception e){
			System.out.println("ReSpecT Error");
		}
		return coda;
	}
	
}
